@@ -12,24 +12,85 @@
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 #   Lesser General Public License for more details.
 
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
+#   You should have received a copy of the GNU Lesser General Public License
+#   along with this library; if not, write to the Free Software Foundation,
+#   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 import re
 
-class _Feature(object):
-    """ Base class """
+
+class _GeoObject(object):
+    """Base Class for Geometry, Feature, and FeatureCollection"""
+
+    def __repr__(self):
+        if self._type == 'Point':
+            return("Point({0}, {1})".format(self.x, self.y))
+        elif self._type == 'LineString':
+            instance = "LineString Instance"
+            qty = len(self.coords)
+            return "<{0} {1} Coords>".format(instance, qty)
+        elif self._type == 'LinearRing':
+            instance = "LinearRing Instance"
+            qty = len(self.coords)
+            return "<{0} {1} Coords>".format(instance, qty)
+        elif self._type == 'Polygon':
+            instance = "Polygon Instance"
+            inter_qty = len(self._interiors)
+            exter_qty = len(self._exterior.coords)
+            return "<{0} {1} Interior {2} Exterior>".format(
+                instance, inter_qty, exter_qty)
+        elif self._type == 'MultiPoint':
+            instance = "MultiPoint Instance"
+            qty = len(self)
+            return "<{0} {1} Points>".format(instance, qty)
+        elif self._type == 'MultiLineString':
+            instance = "MultiLineString Instance"
+            qty = len(self)
+            bounds = self.bounds
+            return "<{0} {1} Lines {2} bbox>".format(instance, qty, bounds)
+        elif self._type == 'MultiPolygon':
+            instance = "MultiPolygon Instance"
+            qty = len(self)
+            bounds = self.bounds
+            return "<{0} {1} Polygons {2} bbox>".format(instance, qty, bounds)
+        elif self._type == 'GeometryCollection':
+            instance = "GeometryCollection Instance"
+            qty = len(self)
+            bounds = self.bounds
+            return "<{0} {1} Geometries {2} bbox>".format(
+                instance, qty, bounds)
+        elif self._type == 'Feature':
+            instance = "Feature Instance"
+            geometry = self._geometry._type
+            properties = len(self._properties)
+            return "<{0} {1} geometry {2} properties>".format(instance,
+                                                              geometry,
+                                                              properties)
+        elif self._type == 'FeatureCollection':
+            instance = 'FeatureCollection Instance'
+            qty = len(self)
+            bounds = self.bounds
+            return '<{0} {1} Features {2} bbox>'.format(instance, qty, bounds)
+
+        else:
+            return object.__repr__(self)
+
+
+class _Geometry(_GeoObject):
+    """
+    Base Class for geometry objects.
+
+    Inherits from GeoObject
+    """
     _type = None
     _coordinates = ()
 
     @property
     def __geo_interface__(self):
         return {
-                'type': self._type,
-                'coordinates': tuple(self._coordinates)
-                }
+            'type': self._type,
+            'coordinates': tuple(self._coordinates)
+            }
 
     def __str__(self):
         return self.to_wkt()
@@ -50,9 +111,68 @@ class _Feature(object):
         raise NotImplementedError
 
 
-class Point(_Feature):
+class Feature(_GeoObject):
     """
-    A zero dimensional feature
+    Aggregates a geometry instance with associated user-defined properties.
+
+    Attributes
+    ~~~~~~~~~~~
+    geometry : object
+        A geometry instance
+    properties : dict
+        A dictionary linking field keys with values
+        associated with geometry instance
+
+    Example
+    ~~~~~~~~
+
+     >>> p = Point(1.0, -1.0)
+     >>> props = {'Name': 'Sample Point', 'Other': 'Other Data'}
+     >>> a = Feature(p, props)
+     >>> a.properties
+     {'Name': 'Sample Point', 'Other': 'Other Data'}
+      >>> a.properties['Name']
+     'Sample Point'
+      """
+
+    _type = 'Feature'
+    _properties = None
+    _geometry = None
+    _feature_id = None
+
+    def __init__(self, geometry, properties={}, feature_id=None, *kwargs):
+        self._geometry = geometry
+        self._properties = properties
+        self._feature_id = feature_id
+
+    @property
+    def id(self):
+        return self._feature_id
+
+    @property
+    def geometry(self):
+        return self._geometry
+
+    @property
+    def properties(self):
+        return self._properties
+
+    @property
+    def __geo_interface__(self):
+        geo_interface = {'type': self._type,
+                         'geometry': self._geometry.__geo_interface__,
+                         'properties': self._properties
+                         }
+        if self._feature_id is None:
+            return geo_interface
+        else:
+            geo_interface['id'] = self._feature_id
+            return geo_interface
+
+
+class Point(_Geometry):
+    """
+    A zero dimensional geometry
 
     A point has zero length and zero area.
 
@@ -84,14 +204,16 @@ class Point(_Feature):
 
         1) 1 parameter: this must satisfy the __geo_interface__ protocol
             or be a tuple or list of x, y, [z]
-        2) 2 or more parameters: x, y, [z] : float
+        2) 2 or 3 parameters: x, y, [z] : float
             Easting, northing, and elevation.
         """
         self._coordinates = ()
         if len(args) == 1:
             if hasattr(args[0], '__geo_interface__'):
                 if args[0].__geo_interface__['type'] == 'Point':
-                    self._coordinates = list(args[0].__geo_interface__['coordinates'])
+                    self._coordinates = list(
+                        args[0].__geo_interface__['coordinates']
+                    )
                 else:
                     raise TypeError
             else:
@@ -109,7 +231,6 @@ class Point(_Feature):
         else:
             raise ValueError
 
-
     @property
     def x(self):
         """Return x coordinate."""
@@ -142,17 +263,16 @@ class Point(_Feature):
         else:
             raise TypeError
 
-
     @property
     def bounds(self):
         return tuple(self._coordinates + self._coordinates)
 
-
     def to_wkt(self):
-        return self._type.upper() + ' ' + str(tuple(self._coordinates)).replace(',','')
+        coords = str(tuple(self._coordinates)).replace(',', '')
+        return self._type.upper() + ' ' + coords
 
 
-class LineString(_Feature):
+class LineString(_Geometry):
     """
     A one-dimensional figure comprising one or more line segments
 
@@ -171,11 +291,9 @@ class LineString(_Feature):
     def __geo_interface__(self):
         if self._type and self._geoms:
             return {
-                    'type': self._type,
-                    'coordinates': tuple(self.coords)
-                    }
-
-
+                'type': self._type,
+                'coordinates': tuple(self.coords)
+            }
 
     def __init__(self, coordinates):
         """
@@ -244,7 +362,7 @@ class LineString(_Feature):
             raise ValueError
 
     def to_wkt(self):
-        wc = [ ' '.join([str(x) for x in c]) for c in self.coords]
+        wc = [' '.join([str(x) for x in c]) for c in self.coords]
         return self._type.upper() + ' (' + ', '.join(wc) + ')'
 
     @property
@@ -261,9 +379,10 @@ class LineString(_Feature):
                 maxy = max(coord[1], maxy)
             return (minx, miny, maxx, maxy)
 
+
 class LinearRing(LineString):
     """
-    A closed one-dimensional feature comprising one or more line segments
+    A closed one-dimensional geometry comprising one or more line segments
 
     A LinearRing that crosses itself or touches itself at a single point is
     invalid and operations on it may fail.
@@ -277,7 +396,6 @@ class LinearRing(LineString):
         if self._geoms[0].coords != self._geoms[-1].coords:
             self._geoms.append(self._geoms[0])
 
-
     @property
     def coords(self):
         if self._geoms[0].coords == self._geoms[-1].coords:
@@ -304,13 +422,13 @@ class LinearRing(LineString):
             self._geoms = self._geoms[::-1]
 
 
-class Polygon(_Feature):
+class Polygon(_Geometry):
     """
     A two-dimensional figure bounded by a linear ring
 
     A polygon has a non-zero area. It may have one or more negative-space
     "holes" which are also bounded by linear rings. If any rings cross each
-    other, the feature is invalid and operations on it may fail.
+    other, the geometry is invalid and operations on it may fail.
 
     Attributes
     ----------
@@ -332,13 +450,12 @@ class Polygon(_Feature):
             return {
                 'type': self._type,
                 'coordinates': tuple(coords)
-                }
+            }
         elif self._exterior:
             return {
                 'type': self._type,
                 'coordinates': (self._exterior.coords,)
-
-                }
+            }
 
     def __init__(self, shell, holes=None):
         """
@@ -380,7 +497,7 @@ class Polygon(_Feature):
             elif gi['type'] == 'Polygon':
                 self._exterior = LinearRing(gi['coordinates'][0])
                 if len(gi['coordinates']) > 1:
-                    #XXX should the holes passed if any be ignored
+                    # XXX should the holes passed if any be ignored
                     # or added to the polygon?
                     self._interiors = []
                     for hole in gi['coordinates'][1:]:
@@ -400,7 +517,6 @@ class Polygon(_Feature):
         else:
             raise TypeError
 
-
     @property
     def exterior(self):
         if self._exterior is not None:
@@ -418,19 +534,16 @@ class Polygon(_Feature):
         if self.exterior:
             return self.exterior.bounds
 
-
     def to_wkt(self):
-        ec = '(' + ', '.join(
-            [ ' '.join([str(x) for x in c]) for c in self.exterior.coords]
-                            ) + ')'
+        ext = [' '.join([str(x) for x in c]) for c in self.exterior.coords]
+        ec = ('(' + ', '.join(ext) + ')')
         ic = ''
         for interior in self.interiors:
             ic += ',(' + ', '.join(
-            [ ' '.join([str(x) for x in c]) for c in interior.coords]
-                            ) + ')'
+                [' '.join([str(x) for x in c]) for c in interior.coords]
+            ) + ')'
         return self._type.upper() + '(' + ec + ic + ')'
 
-
     def _set_orientation(self, clockwise=False, exterior=True, interiors=True):
         """ sets the orientation of the coordinates in
         clockwise or counterclockwise (default) order"""
@@ -441,8 +554,7 @@ class Polygon(_Feature):
                 interior._set_orientation(clockwise)
 
 
-
-class MultiPoint(_Feature):
+class MultiPoint(_Geometry):
     """A collection of one or more points
 
     Attributes
@@ -459,7 +571,7 @@ class MultiPoint(_Feature):
         return {
             'type': self._type,
             'coordinates': tuple([g.coords[0] for g in self._geoms])
-            }
+        }
 
     def __init__(self, points):
         """
@@ -535,20 +647,13 @@ class MultiPoint(_Feature):
                 maxy = max(geom.coords[0][1], maxy)
             return (minx, miny, maxx, maxy)
 
-
     def unique(self):
         """ Make Points unique, delete duplicates """
-        coords = []
-        for geom in self.geoms:
-            coords.append(geom.coords)
-        coords = list(set(coords))
-        self._geoms = []
-        for coord in coords:
-            p = Point(coord[0])
-            self._geoms.append(p)
+        coords = [geom.coords for geom in self.geoms]
+        self._geoms = [Point(coord[0]) for coord in set(coords)]
 
     def to_wkt(self):
-        wc = [ ' '.join([str(x) for x in c.coords[0]]) for c in self.geoms]
+        wc = [' '.join([str(x) for x in c.coords[0]]) for c in self.geoms]
         return self._type.upper() + '(' + ', '.join(wc) + ')'
 
     def __len__(self):
@@ -558,8 +663,7 @@ class MultiPoint(_Feature):
             return 0
 
 
-
-class MultiLineString(_Feature):
+class MultiLineString(_Geometry):
     """
     A collection of one or more line strings
 
@@ -577,8 +681,10 @@ class MultiLineString(_Feature):
     def __geo_interface__(self):
         return {
             'type': self._type,
-            'coordinates': tuple(tuple(c for c in g.coords) for g in self.geoms)
-            }
+            'coordinates': tuple(
+                tuple(c for c in g.coords) for g in self.geoms
+            )
+        }
 
     def __init__(self, lines):
         """
@@ -606,13 +712,14 @@ class MultiLineString(_Feature):
                 l = LineString(gi['coordinates'])
                 self._geoms.append(l)
             elif gi['type'] == 'MultiLineString':
-                for line in  gi['coordinates']:
+                for line in gi['coordinates']:
                     l = LineString(line)
                     self._geoms.append(l)
             else:
                 raise TypeError
         else:
             raise TypeError
+
     @property
     def geoms(self):
         return tuple(self._geoms)
@@ -631,16 +738,14 @@ class MultiLineString(_Feature):
                 maxy = max(geom.bounds[3], maxy)
             return (minx, miny, maxx, maxy)
 
-
-
     def to_wkt(self):
         wc = '(' + ', '.join(
-            [ ' '.join([str(x) for x in c]) for c in self.geoms[0].coords]
-                            ) + ')'
+            [' '.join([str(x) for x in c]) for c in self.geoms[0].coords]
+        ) + ')'
         for lx in self.geoms[1:]:
             wc += ',(' + ', '.join(
-            [ ' '.join([str(x) for x in c]) for c in lx.coords]
-                            ) + ')'
+                [' '.join([str(x) for x in c]) for c in lx.coords]
+            ) + ')'
         return self._type.upper() + '(' + wc + ')'
 
     def __len__(self):
@@ -649,7 +754,8 @@ class MultiLineString(_Feature):
         else:
             return 0
 
-class MultiPolygon(_Feature):
+
+class MultiPolygon(_Geometry):
     """A collection of one or more polygons
 
     If component polygons overlap the collection is `invalid` and some
@@ -675,7 +781,7 @@ class MultiPolygon(_Feature):
         return {
             'type': self._type,
             'coordinates': tuple(allcoords)
-            }
+        }
 
     def __init__(self, polygons):
         """
@@ -690,12 +796,12 @@ class MultiPolygon(_Feature):
         -------
         Construct a collection from a sequence of coordinate tuples
 
-          >>> ob = MultiPolygon( [
+          >>> ob = MultiPolygon([
           ...     (
           ...     ((0.0, 0.0), (0.0, 1.0), (1.0, 1.0), (1.0, 0.0)),
-          ...     [((0.1,0.1), (0.1,0.2), (0.2,0.2), (0.2,0.1))]
-          ...     )
-          ... ] )
+          ...     [((0.1, 0.1), (0.1, 0.2), (0.2, 0.2), (0.2, 0.1))]
+          ...)
+          ...])
           >>> len(ob.geoms)
           1
           >>> type(ob.geoms[0]) == Polygon
@@ -744,18 +850,17 @@ class MultiPolygon(_Feature):
                 maxy = max(geom.bounds[3], maxy)
             return (minx, miny, maxx, maxy)
 
-
     def to_wkt(self):
         pc = ''
         for geom in self.geoms:
             ec = '(' + ', '.join(
-                [ ' '.join([str(x) for x in c]) for c in geom.exterior.coords]
-                                ) + ')'
+                [' '.join([str(x) for x in c]) for c in geom.exterior.coords]
+            ) + ')'
             ic = ''
             for interior in geom.interiors:
                 ic += ',(' + ', '.join(
-                [ ' '.join([str(x) for x in c]) for c in interior.coords]
-                                ) + ')'
+                    [' '.join([str(x) for x in c]) for c in interior.coords]
+                ) + ')'
             pc += '(' + ec + ic + ')'
         return self._type.upper() + '(' + pc + ')'
 
@@ -772,8 +877,10 @@ class MultiPolygon(_Feature):
         else:
             return 0
 
-class GeometryCollection(_Feature):
-    """A heterogenous collection of geometries
+
+class GeometryCollection(_Geometry):
+    """A heterogenous collection of geometries (Points, LineStrings,
+       LinearRings, and Polygons)
 
     Attributes
     ----------
@@ -781,32 +888,45 @@ class GeometryCollection(_Feature):
         A sequence of geometry instances
 
     Please note:
-    GEOMETRYCOLLECTION isn't supported by the Shapefile format.
-    And this sub-class isn't generally supported by ordinary GIS sw (viewers and so on).
-    So it's very rarely used in the real GIS professional world.
+    GEOMETRYCOLLECTION isn't supported by the Shapefile format. And this sub-
+    class isn't generally supported by ordinary GIS sw (viewers and so on). So
+    it's very rarely used in the real GIS professional world.
+
+    Example
+    -------
+
+    Initialize Geometries and construct a GeometryCollection
+
+    >>> from pygeoif import geometry
+    >>> p = geometry.Point(1.0, -1.0)
+    >>> p2 = geometry.Point(1.0, -1.0)
+    >>> geoms = [p, p2]
+    >>> c = geometry.GeometryCollection(geoms)
+    >>> c.__geo_interface__
+    {'type': 'GeometryCollection',
+    'geometries': [{'type': 'Point', 'coordinates': (1.0, -1.0)},
+    {'type': 'Point', 'coordinates': (1.0, -1.0)}]}
     """
     _type = 'GeometryCollection'
     _geoms = None
 
+    _allowed_geomtries = (Point, LineString, LinearRing, Polygon)
+
     @property
     def __geo_interface__(self):
         gifs = []
         for geom in self._geoms:
             gifs.append(geom.__geo_interface__)
-        return {'type': self._type, 'geometries': tuple(gifs) }
+        return {'type': self._type, 'geometries': gifs}
 
-    def __init__(self, features):
+    def __init__(self, geometries):
         self._geoms = []
-        if isinstance(features, (list, tuple)):
-            for feature in features:
-                if isinstance(feature,
-                        (Point, LineString, LinearRing, Polygon,
-                        MultiPoint, MultiLineString, MultiPolygon)):
-                    self._geoms.append(feature)
-                elif isinstance(as_shape(feature),
-                        (Point, LineString, LinearRing, Polygon,
-                        MultiPoint, MultiLineString, MultiPolygon)):
-                    self._geoms.append(as_shape(feature))
+        if isinstance(geometries, (list, tuple)):
+            for geometry in geometries:
+                if isinstance(geometry, self._allowed_geomtries):
+                    self._geoms.append(geometry)
+                elif isinstance(as_shape(geometry), self._allowed_geomtries):
+                    self._geoms.append(as_shape(geometry))
                 else:
                     raise ValueError
         else:
@@ -815,13 +935,11 @@ class GeometryCollection(_Feature):
     @property
     def geoms(self):
         for geom in self._geoms:
-            if isinstance(geom, (Point, LineString, LinearRing, Polygon,
-                                MultiPoint, MultiLineString, MultiPolygon)):
+            if isinstance(geom, self._allowed_geomtries):
                 yield geom
             else:
                 raise ValueError("Illegal geometry type.")
 
-
     @property
     def bounds(self):
         if self._geoms:
@@ -848,6 +966,92 @@ class GeometryCollection(_Feature):
         else:
             return 0
 
+    def __iter__(self):
+        return iter(self._geoms)
+
+
+class FeatureCollection(_GeoObject):
+    """A heterogenous collection of Features
+
+    Attributes
+    ----------
+    features : sequence
+        A sequence of feature instances
+
+
+    Example
+    -------
+
+    >>> from pygeoif import geometry
+    >>> p = geometry.Point(1.0, -1.0)
+    >>> props = {'Name': 'Sample Point', 'Other': 'Other Data'}
+    >>> a = geometry.Feature(p, props)
+    >>> p2 = geometry.Point(1.0, -1.0)
+    >>> props2 = {'Name': 'Sample Point2', 'Other': 'Other Data2'}
+    >>> b = geometry.Feature(p2, props2)
+    >>> features = [a, b]
+    >>> c = geometry.FeatureCollection(features)
+    >>> c.__geo_interface__
+    {'type': 'FeatureCollection',
+     'features': [{'geometry': {'type': 'Point', 'coordinates': (1.0, -1.0)},
+     'type': 'Feature',
+     'properties': {'Other': 'Other Data', 'Name': 'Sample Point'}},
+    {'geometry': {'type': 'Point', 'coordinates': (1.0, -1.0)},
+     'type': 'Feature',
+     'properties': {'Other': 'Other Data2', 'Name': 'Sample Point2'}}]}
+    """
+    _type = 'FeatureCollection'
+    _features = None
+
+    @property
+    def __geo_interface__(self):
+        gifs = []
+        for feature in self._features:
+            gifs.append(feature.__geo_interface__)
+        return {'type': self._type, 'features': gifs}
+
+    def __init__(self, features):
+        self._features = []
+        if isinstance(features, (list, tuple)):
+            for feature in features:
+                if isinstance(feature, Feature):
+                    self._features.append(feature)
+                else:
+                    raise ValueError
+        else:
+            raise TypeError
+
+    @property
+    def features(self):
+        for feature in self._features:
+            if isinstance(feature, Feature):
+                yield feature
+            else:
+                raise ValueError("Illegal type.")
+
+    @property
+    def bounds(self):
+        if self._features:
+            minx = self._features[0].geometry.bounds[0]
+            miny = self._features[0].geometry.bounds[1]
+            maxx = self._features[0].geometry.bounds[2]
+            maxy = self._features[0].geometry.bounds[3]
+            for feature in self.features:
+                minx = min(feature.geometry.bounds[0], minx)
+                miny = min(feature.geometry.bounds[1], miny)
+                maxx = max(feature.geometry.bounds[2], maxx)
+                maxy = max(feature.geometry.bounds[3], maxy)
+            return (minx, miny, maxx, maxy)
+
+    def __len__(self):
+        if self._features:
+            return len(self._features)
+        else:
+            return 0
+
+    def __iter__(self):
+        return iter(self._features)
+
 
 def signed_area(coords):
     """Return the signed area enclosed by a ring using the linear time
@@ -881,35 +1085,49 @@ def orient(polygon, sign=1.0):
     return Polygon(rings[0], rings[1:])
 
 
-def as_shape(feature):
-    """ creates a pygeoif feature from an object that
+def as_shape(geometry):
+    """ creates a pygeoif geometry from an object that
     provides the __geo_interface__ or a dictionary that
     is __geo_interface__ compatible"""
     gi = None
-    if isinstance(feature, dict):
-        if ('coordinates' in feature) and ('type' in feature):
-            gi = feature
-        elif (feature['type'] == 'GeometryCollection') and ('geometries' in feature):
-            gi = feature
-    elif hasattr(feature, '__geo_interface__'):
-        gi = feature.__geo_interface__
+    if isinstance(geometry, dict):
+        is_geometryCollection = geometry['type'] == 'GeometryCollection'
+        is_feature = geometry['type'] == 'Feature'
+        if ('coordinates' in geometry) and ('type' in geometry):
+            gi = geometry
+        elif is_geometryCollection and 'geometries' in geometry:
+            gi = geometry
+        elif is_feature:
+            gi = geometry
+    elif hasattr(geometry, '__geo_interface__'):
+        gi = geometry.__geo_interface__
     else:
         try:
             # maybe we can convert it into a valid __geo_interface__ dict
-            cdict = dict(feature)
+            cdict = dict(geometry)
+            is_geometryCollection = cdict['type'] == 'GeometryCollection'
             if ('coordinates' in cdict) and ('type' in cdict):
                 gi = cdict
-            elif (cdict['type'] == 'GeometryCollection') and ('geometries' in cdict):
+            elif is_geometryCollection and 'geometries' in cdict:
                 gi = cdict
         except:
             pass
     if gi:
         ft = gi['type']
         if ft == 'GeometryCollection':
-            features = []
+            geometries = []
             for fi in gi['geometries']:
+                geometries.append(as_shape(fi))
+            return GeometryCollection(geometries)
+        if ft == 'Feature':
+            return Feature(as_shape(gi['geometry']),
+                           properties=gi.get('properties', {}),
+                           feature_id=gi.get('id', None))
+        if ft == 'FeatureCollection':
+            features = []
+            for fi in gi['features']:
                 features.append(as_shape(fi))
-            return GeometryCollection(features)
+            return FeatureCollection(features)
         coords = gi['coordinates']
         if ft == 'Point':
             return Point(coords)
@@ -935,10 +1153,13 @@ def as_shape(feature):
 
 
 wkt_regex = re.compile(r'^(SRID=(?P<srid>\d+);)?'
-    r'(?P<wkt>'
-    r'(?P<type>POINT|LINESTRING|LINEARRING|POLYGON|MULTIPOINT|MULTILINESTRING|MULTIPOLYGON|GEOMETRYCOLLECTION)'
-    r'[ACEGIMLONPSRUTYZ\d,\.\-\(\) ]+)$',
-    re.I)
+                       r'(?P<wkt>'
+                       r'(?P<type>POINT|LINESTRING|LINEARRING|POLYGON|'
+                       r'MULTIPOINT|MULTILINESTRING|MULTIPOLYGON|'
+                       r'GEOMETRYCOLLECTION)'
+                       r'[ACEGIMLONPSRUTYZ\d,\.\-\(\) ]+)$',
+                       re.I
+                       )
 
 gcre = re.compile(r'POINT|LINESTRING|LINEARRING|POLYGON')
 
@@ -946,6 +1167,7 @@ outer = re.compile("\((.+)\)")
 inner = re.compile("\([^)]*\)")
 mpre = re.compile("\(\((.+?)\)\)")
 
+
 def from_wkt(geo_str):
     """
     Create a geometry from its WKT representation
@@ -984,7 +1206,7 @@ def from_wkt(geo_str):
         coords = []
         for coord in coords1:
             if '(' in coord:
-                coord = coord[coord.find('(') +1 : coord.rfind(')')]
+                coord = coord[coord.find('(') + 1: coord.rfind(')')]
             coords.append(coord.strip())
         return MultiPoint([c.split() for c in coords])
     elif ftype == 'MULTILINESTRING':
@@ -999,7 +1221,7 @@ def from_wkt(geo_str):
             if len(polygon) < 3:
                 continue
             coords = []
-            for interior in inner.findall('(' +polygon +')'):
+            for interior in inner.findall('(' + polygon + ')'):
                 coords.append((interior[1:-1]).split(','))
             if len(coords) > 1:
                 # we have a polygon with holes
@@ -1011,20 +1233,17 @@ def from_wkt(geo_str):
             polygons.append(Polygon([c.split() for c in coords[0]], exteriors))
         return MultiPolygon(polygons)
     elif ftype == 'GEOMETRYCOLLECTION':
-        gc_type = gcre.findall(coordinates)
+        gc_types = gcre.findall(coordinates)
         gc_coords = gcre.split(coordinates)[1:]
-        assert(len(gc_type)==len(gc_coords))
-        features = []
-        for i in range(0,len(gc_type)):
-            gc_wkt = gc_type[i] + gc_coords[i][:gc_coords[i].rfind(')') +1]
-            features.append(from_wkt(gc_wkt))
-        return GeometryCollection(features)
+        assert(len(gc_types) == len(gc_coords))
+        geometries = []
+        for (gc_type, gc_coord) in zip(gc_types, gc_coords):
+            gc_wkt = gc_type + gc_coord[:gc_coord.rfind(')') + 1]
+            geometries.append(from_wkt(gc_wkt))
+        return GeometryCollection(geometries)
     else:
         raise NotImplementedError
 
+
 def mapping(ob):
     return ob.__geo_interface__
-
-
-
-
